@@ -30,12 +30,16 @@ import numpy
 
 _LOGGER = logging.getLogger(__name__)
 
-_Measurement = collections.namedtuple(
-    "measurement",
+Measurement = collections.namedtuple(
+    "Measurement",
     ["decoding_timestamp", "temperature_degrees_celsius", "relative_humidity"],
 )
 
 
+class _UnexpectedPacketLengthError(ValueError):
+    pass
+
+
 class DecodeError(ValueError):
     pass
 
@@ -54,7 +58,7 @@ class FT017TH:
     _MESSAGE_REPEATS = 3
 
     @classmethod
-    def _parse_message(cls, bits) -> _Measurement:
+    def _parse_message(cls, bits) -> Measurement:
         assert bits.shape == (cls._MESSAGE_LENGTH_BITS,), bits.shape
         if (bits[:8] != 1).any():
             raise DecodeError("invalid prefix in message: {}".format(bits))
@@ -81,7 +85,7 @@ class FT017TH:
             relative_humidity * 100,
             bits[56:],  # checksum?
         )
-        return _Measurement(
+        return Measurement(
             decoding_timestamp=_now_local(),
             temperature_degrees_celsius=temperature_degrees_celsius,
             relative_humidity=relative_humidity,
@@ -90,7 +94,7 @@ class FT017TH:
     @classmethod
     def _parse_transmission(
         cls, signal: numpy.ndarray  # dtype=numpy.uint8
-    ) -> _Measurement:
+    ) -> Measurement:
         bits = numpy.unpackbits(signal)[
             : cls._MESSAGE_LENGTH_BITS * cls._MESSAGE_REPEATS
         ]  # bitorder='big'
@@ -104,64 +108,96 @@ class FT017TH:
 
     _SYNC_WORD = bytes([255, 168])  # 168 might be sender-specific
 
-    def __init__(self):
-        self.transceiver = cc1101.CC1101()
+    def __init__(self, unlock_spi_device: bool = False):
+        """
+        unlock_spi_device:
+            If True, flock on SPI device file /dev/spidev0.0
+            will be released after configuring the transceiver.
+            Useful if another process (infrequently) accesses
+            the transceiver simultaneously.
+        """
+        self._unlock_spi_device = unlock_spi_device
+        self._transceiver = cc1101.CC1101(lock_spi_device=True)
+        self._transmission_length_bytes = math.ceil(
+            self._MESSAGE_LENGTH_BITS * self._MESSAGE_REPEATS / 8
+        )
 
     def _configure_transceiver(self):
-        self.transceiver.set_base_frequency_hertz(433.945e6)
-        self.transceiver.set_symbol_rate_baud(2048)
-        self.transceiver.set_sync_mode(
+        self._transceiver.set_base_frequency_hertz(433.945e6)
+        self._transceiver.set_symbol_rate_baud(2048)
+        self._transceiver.set_sync_mode(
             cc1101.SyncMode.TRANSMIT_16_MATCH_15_BITS,
             _carrier_sense_threshold_enabled=True,
         )
-        self.transceiver.set_sync_word(self._SYNC_WORD)
-        self.transceiver.disable_checksum()
-        self.transceiver.enable_manchester_code()
-        self.transceiver.set_packet_length_mode(cc1101.PacketLengthMode.FIXED)
-        self.transceiver.set_packet_length_bytes(
-            math.ceil(self._MESSAGE_LENGTH_BITS * self._MESSAGE_REPEATS / 8)
-            - len(self._SYNC_WORD)
+        self._transceiver.set_sync_word(self._SYNC_WORD)
+        self._transceiver.disable_checksum()
+        self._transceiver.enable_manchester_code()
+        self._transceiver.set_packet_length_mode(cc1101.PacketLengthMode.FIXED)
+        self._transceiver.set_packet_length_bytes(
+            self._transmission_length_bytes - len(self._SYNC_WORD)
         )
         # pylint: disable=protected-access; version pinned
-        self.transceiver._set_filter_bandwidth(mantissa=3, exponent=3)
+        self._transceiver._set_filter_bandwidth(mantissa=3, exponent=3)
 
     def _receive_packet(
         self,
     ) -> typing.Optional[
         cc1101._ReceivedPacket  # pylint: disable=protected-access; version pinned
     ]:
-        self.transceiver._enable_receive_mode()  # pylint: disable=protected-access; version pinned
+        self._transceiver._enable_receive_mode()  # pylint: disable=protected-access; version pinned
         time.sleep(0.05)
         while (
-            self.transceiver.get_marc_state()
+            self._transceiver.get_marc_state()
             == cc1101.MainRadioControlStateMachineState.RX
         ):
             time.sleep(8.0)  # transmits approx once per minute
         return (
             # pylint: disable=protected-access; version pinned
-            self.transceiver._get_received_packet()
+            self._transceiver._get_received_packet()
         )
 
-    def receive(self) -> typing.Iterator[_Measurement]:
-        with self.transceiver:
-            self._configure_transceiver()
-            _LOGGER.debug(
-                "%s, filter_bandwidth=%.0fkHz",
-                self.transceiver,
-                # pylint: disable=protected-access; version pinned
-                self.transceiver._get_filter_bandwidth_hertz() / 1000,
-            )
-            while True:
-                packet = self._receive_packet()
-                if packet:
-                    _LOGGER.debug("%s", packet)
-                    try:
-                        yield self._parse_transmission(
-                            numpy.frombuffer(
-                                self._SYNC_WORD + packet.data, dtype=numpy.uint8
-                            )
-                        )
-                    except DecodeError as exc:
-                        _LOGGER.debug(
-                            "failed to decode %s: %s", packet, str(exc), exc_info=exc
-                        )
+    def _receive_measurement(self) -> typing.Optional[Measurement]:
+        packet = self._receive_packet()
+        if not packet:
+            return None
+        signal = numpy.frombuffer(self._SYNC_WORD + packet.data, dtype=numpy.uint8)
+        if signal.shape != (self._transmission_length_bytes,):
+            raise _UnexpectedPacketLengthError()
+        try:
+            return self._parse_transmission(signal)
+        except DecodeError as exc:
+            _LOGGER.debug("failed to decode %s: %s", packet, str(exc), exc_info=exc)
+        return None
+
+    _LOCK_WAIT_START_SECONDS = 2
+    _LOCK_WAIT_FACTOR = 2
+
+    def receive(self) -> typing.Iterator[Measurement]:
+        lock_wait_seconds = self._LOCK_WAIT_START_SECONDS
+        while True:
+            try:
+                with self._transceiver:
+                    self._configure_transceiver()
+                    _LOGGER.debug(
+                        "%s, filter_bandwidth=%.0fkHz",
+                        self._transceiver,
+                        # pylint: disable=protected-access; version pinned
+                        self._transceiver._get_filter_bandwidth_hertz() / 1000,
+                    )
+                    if self._unlock_spi_device:
+                        self._transceiver.unlock_spi_device()
+                        _LOGGER.debug("unlocked SPI device")
+                    while True:
+                        measurement = self._receive_measurement()
+                        if measurement:
+                            yield measurement
+                            lock_wait_seconds = self._LOCK_WAIT_START_SECONDS
+            except _UnexpectedPacketLengthError:
+                _LOGGER.info(
+                    "unexpected packet length;"
+                    " reconfiguring as transceiver was potentially accessed by another process"
+                )
+            except BlockingIOError:
+                _LOGGER.info("SPI device locked, waiting %d seconds", lock_wait_seconds)
+                time.sleep(lock_wait_seconds)
+                lock_wait_seconds *= self._LOCK_WAIT_FACTOR
